;________________________________________________________________________________________
;MODEL
;Organizational Learning in Innovation Networks:
;Exploring the Role of Cognitive Distance and Absorptive Capacity. An Agent-Based Model
;----------------------------------------------------------------------------------------
;Author: Sandra Schmid
;________________________________________________________________________________________


;----------------------------------------------
;BREEDS
;----------------------------------------------

breed [firms firm]


;----------------------------------------------
;STATE VARIABLES
;----------------------------------------------

globals [
  ;network-members             ;Number of firms in the network (interface slider)
  ;acceptable-satisfaction     ;Satisfaction level where firms stop searching for a partner with a better cognitive fit (interface slider)
  number-of-dyads              ;Variable which counts the total number of partnerships which have been formed throughout the life cycle of the network
  number-of-satisfied-dyads    ;Variable which counts the number of satisfied partnerships over the life cycle of the network
  leaving-firms                ;Variable which counts in each time step the number of firms which leave the network, because they are frustrated
  leaving-firms-total          ;Variable which counts the overall number of firms leaving the network during the life cycle of the network
  new-entrants                 ;Variable which counts the new firms entering the network (over the whole lifetime of the network)
  ;search-costs-threshold      ;Variable: Number of unsuccessful contacts, i.e. search costs, at which a firm leaves the network (interface slider)
  ;firm-satisfaction?          ;Boolean reporter which decides whether satisfaction is a global variable or a firm variable (interface switch)
  
  ;-Learning characteristics
  ;exploratory-learning-setting ;Variable which determines how exploratory learning is calculated; two options: linear or quadratic (interface chooser)
  ;var-opt-CD                  ;Variable: This is the variability which is allowed as deviation from optimal-cognitive-distance (interface chooser)
  ;AC-increase-rate            ;Variable, determines the rate of learning (effect of novel knowledge creation on prior knowledge (maxAC)) (interface chooser)
  network-knowledge-created-month  ;This variable contains for every tick (month) the total knowledge-created from the network. It is set every round to 0 again
  network-knowledge-created-total  ;This variable contains the total knowledge-created from the network
  median-netw-knowl-month      ;Variable calculates the network's monthly median of novel knowledge created
  ;evaluation-interval         ;Variable, which influences the speed of learning (maxAC increase) (interface chooser)
  time-intervals               ;List which contains the points of time at which a firm's evaluation takes place during an ongoing partnership 
  ]

firms-own [
  partner                      ;Boolean reporter - true if agent has a partner, false otherwise
  partner-list                 ;List which memorizes the last partners
  optimal-cognitive-distance   ;Value which reflects the optimal cognitive distance of a firm
  cognitive-distance-list      ;List which memorizes the values of cognitive distance of the last partnerships
  lower-bound-CD               ;Variable which reports the lower limit of cognitive distance as partner selection criterion
  upper-bound-CD               ;Variable which reports the upper limit of cognitive distance as partner selection criterion
  novelty-value-list           ;List in which every firm memorizes the novelty value inherent in the respective partnership
  absorptive-capacity          ;Variable, displays the firm's absorptive capacity with respect to its current partner
  max-absorptive-capacity      ;Value which reflects the maximum level of absorptive capacity of a firm
  satisfaction                 ;Variable which reflects the firm's satisfaction with the current partner
  satisfied                    ;Boolean reporter - true if partnership is satisfying in terms of satisfaction and cognitive distance measure, false otherwise
  optimal-satisfaction         ;Firms are most satisfied with their relationship, no further improvement possible
  n-satisfying-partners        ;This variable counts the number of partners which were satisfying (over the whole "life" of the firm in the network)
  firm-acceptable-satisfaction ;Variable which reflects the firm's individual acceptable satisfaction level at which it turns to be satisfied; becomes
                               ;only effective if firm-satisfaction? is on
  search-costs                 ;Variable which counts the number of a firm's unsuccessful contacts with another firm without finding a partner (search costs)
                               
  ;-Learning characteristics
  exploratory-learning         ;Variable: Value reflects the firm's ability to recognize novel knowledge
  transformative-learning      ;Variable: Value reflects the firm's ability to assimilate and transform absorbed knowledge
  exploitative-learning        ;Variable: Value reflects the firm's ability to exploit assimilated and transformed knowledge
  dyadic-learning-probability  ;Variable which shows a firm's probability of learning from a specific dyadic relationship
  min-dyadic-learn-prob        ;Variable: firm memorizes its minimal value in dyadic learning probability
  max-dyadic-learn-prob        ;Variable: firm memorizes its maximal value in dyadic learning probability
  max-AC-increase-current      ;Variable: Firms keep record of their current learning success, i.e. increase in max-AC
  max-AC-increase-total        ;Variable which keeps record of a firm's overall learning success (increase in MaxAC) over its lifetime in the network
  
  ;-Collaboration characteristics
  knowledge-created-month      ;Variable: Sum of a firm's monthly created knowledge
  knowledge-created-total      ;Variable: Accumulated sum of a firm's totally created knowledge
  n-rounds-collaboration-total ;Variable: Number of successful collaboration rounds over all satisfying partners
  n-rounds-collaboration-this-dyad  ;Variable: Number of successful collaboration rounds with this partner
  below-mean                   ;Variable, shows how many times firm performs below the network's mean (in this partnership)
  above-mean                   ;Variable, shows how many times firm performs above the network's mean (in this partnership)
  CD-category                  ;String variable which shows the cognitive distance category of a firm in terms of the current dyad
  max-AC-difference-list       ;List in which a firm stores the values of the difference between the max-absorptive-capacity values of itself and its partners
  ]  

links-own [
  ;cognitive-distance          ;Euclidean distance between two partners; specific to this partnership (Reporter)
  ;novelty-value               ;Variable which reflects the novelty value in a relationship according to its cognitive distance; is the same for all firms
                               ;operating at the same cognitive distance (Reporter)
  max-AC-difference            ;Variable which contains the difference between the max-absorptive-capacity values of the firms of a dyad
  ]

patches-own []



;----------------------------------------------
;INITIALIZATION
;----------------------------------------------

;----------------------------------------------
;INITIALIZATION - Model setup (Observer)
;----------------------------------------------

to setup
  clear-all
  
  ;;Set global variables
  ;At the beginning there are no new entrants and no firms have
  ;left the network
  set new-entrants 0
  set leaving-firms 0
  set leaving-firms-total 0
  
  ;All global knowledge variables have starting values of 0
  set network-knowledge-created-month 0
  set network-knowledge-created-total 0
  set median-netw-knowl-month 0
  
  ;Firms are created
  setup-firms

  ;Update values which are needed for plotting
  ;Variable which counts the number of dyads
  set number-of-dyads count links
  set number-of-satisfied-dyads 0
  
  ;Evaluation cycles of firms during an enduring partnership
  if evaluation-interval = "quarterly"
   [set time-intervals [3 6 9 12 15 18 21 24 27 30 33 36 39 42 45 48]]
  if evaluation-interval = "half-year"
    [set time-intervals [6 12 18 24 30 36 42 48]]
  if evaluation-interval = "yearly"
    [set time-intervals [12 24 36 48]]
  if evaluation-interval = "two-year"
    [set time-intervals [24 48]]
  
  ;Prepare output-file for reporting of firms
  write-headings-to-file-satisfied
  write-headings-to-file-frustrated
  
  reset-ticks 
  
end



;----------------------------------------------
;Initialization 1 - "Setup of firms" (Observer)
;----------------------------------------------

to setup-firms
  
  ;Number of firms are created with the characteristics defined in 
  ;subprocess "setup-firm-characteristics" 
  create-firms network-members
   [setup-firm-characteristics]
  
  ;Every firm gets one random partner at the start
  ask firms                                            
  [ifelse partner = true
    [stop]
    [ifelse any? other firms with [partner = false]
      [create-link-to one-of other firms with [partner = false]
        ask links [ask both-ends [set partner true]]]
      [stop]
    ]
  ]
  
  ;Firms update the values of their characteristics
  ;according to the subprocess "update-firm-characteristics"
  ask firms 
   [update-firm-characteristics]

end



;----------------------------------------------
;Initialization 1.1 - "Setup of firm characteristics" 
;(Observer)
;----------------------------------------------

to setup-firm-characteristics
  
  set size 0.5
  set color blue
  set shape "house"
  setxy random-pxcor random-pycor
  ;Random normal distribution of max-absorptive-capacity of firms
  let transit random-normal 14 5
  set max-absorptive-capacity transit
  if transit < 0
    [set max-absorptive-capacity 0]
  if transit > (sqrt ((max-pxcor ^ 2) + (max-pycor ^ 2)))
    [set max-absorptive-capacity (sqrt ((max-pxcor ^ 2) + (max-pycor ^ 2)))]
  
  ;Optimal satisfaction is reached when the difference
  ;between absorptive capacity and novelty value is 0
  set optimal-satisfaction 0
  
  ;If satisfaction is on firm level, the firm's acceptable satisfaction level is randomly distributed and can contain
  ;values ranging from -28 to 28 (rounded to one decimal)
  if firm-satisfaction? = true
    [let possible-range  precision (-28 + random-float 56) 1
      set firm-acceptable-satisfaction possible-range]
  
  ;The point of optimal-cognitive-distance is at the intersection between novelty value and absorptive capacity
  set optimal-cognitive-distance max-absorptive-capacity / 2
  
  set partner false
  set partner-list []
  set cognitive-distance-list []
  set lower-bound-CD 0
  ;Value of max-absorptive-capacity determines upper-bound-CD, this is cognitively the most distant point which is accessible for this firm
  set upper-bound-CD max-absorptive-capacity
  set novelty-value-list []
  ;Satisfaction: -100 is a symbolic value which shows that there is no current value for firm satisfaction (as 0 has a meaning in this context),
  ;or that firm is very unsatisfied
  set satisfaction -100
  set satisfied false
  set search-costs 0
  
  ;;Set learning characteristics
  ;-Exploratory Learning
  set-exploratory-learning
  
  ;-Transformative Learning
  ;random probability ranging from 0.1, 0.2,... to 1
  set transformative-learning precision (0.1 + random-float 0.9) 1
  
  ;-Exploitative Learning
  ;random probability ranging from 0.1, 0.2,... to 1
  set exploitative-learning precision (0.1 + random-float 0.9) 1

  ;Every firms starts in the network with 0 knowledge created and 0 collaboration rounds
  set knowledge-created-month 0
  set knowledge-created-total 0
  set n-rounds-collaboration-total 0
  set n-rounds-collaboration-this-dyad 0
  set below-mean 0
  set above-mean 0
  
  ;Dyadic learning
  set dyadic-learning-probability 0
  set min-dyadic-learn-prob 0
  set max-dyadic-learn-prob 0
  set CD-category "NA"
  set max-AC-difference-list []
  
  ;Firms start with a learning success (increase in Max-AC) of 0
  set max-AC-increase-current 0
  set max-AC-increase-total 0
  
  ;Number of satisfying partners
  set n-satisfying-partners 0
 
  
end

;---------------------------------------------
;Initialization 1.1.a Exploratory Learning 
;(Observer)
;---------------------------------------------
to set-exploratory-learning
  if exploratory-learning-setting = "linear"
  [
    ;Linear relationship between max-AC and exploratory-learning
    ;rounded to one decimal
    let slope-linear ( 1 / (sqrt ((max-pxcor ^ 2) + (max-pycor ^ 2))))
    set exploratory-learning precision (slope-linear * max-absorptive-capacity) 1
  ]
  if exploratory-learning-setting = "quadratic"
  [
    ;Quadratic relationship between max-AC and exploratory-learning
    ;rounded to one decimal
    let term-quadratic ( 1 / (sqrt ((max-pxcor ^ 2) + (max-pycor ^ 2))))
    set exploratory-learning precision ((term-quadratic * max-absorptive-capacity) ^ 2) 1
  ]
end



;----------------------------------------------
;Initialization 1.2 - "Update values of firm 
;characteristics" (Turtle)
;----------------------------------------------

to update-firm-characteristics
  
  ;Update partner-list (partner memory)
  ;Firms which learned, do not update the partner memory
  ifelse max-AC-increase-current > 0
  []
  [
    if any? my-in-links
    [let inbound one-of my-in-links
      let transit [other-end] of inbound
      set partner-list lput transit partner-list]
    if any? my-out-links
    [let outbound one-of my-out-links
      let transit [other-end] of outbound
      set partner-list lput transit partner-list]
  ]
  
  ;Calculate the absolute difference between max-absorptive-capacity values of firms of a dyad
  if any? my-out-links
    [let max-AC1 max-absorptive-capacity
      let outbound one-of my-out-links
      ask [other-end] of outbound 
      [let max-AC2 max-absorptive-capacity
        let transit abs (max-AC1 - max-AC2)
        ask outbound [set max-AC-difference transit]
      ]]
    
  ;Update cognitive distance
  ;Firms which learned, do not update their cognitive distance
  ifelse max-AC-increase-current > 0
  []
  [
    if any? my-in-links
    [let inbound one-of my-in-links
      let transit [cognitive-distance] of inbound
      set cognitive-distance-list lput transit cognitive-distance-list]
    if any? my-out-links
    [let outbound one-of my-out-links
      let transit [cognitive-distance] of outbound
      set cognitive-distance-list lput transit cognitive-distance-list]
  ]
  
  ;Define partner selection criteria
  ;Firms which learned update their cognitive distance selection criteria differently
  ifelse max-AC-increase-current > 0
  []
  [
    if partner = true 
    [
      ;Lower limit
      if last cognitive-distance-list < optimal-cognitive-distance
      [set lower-bound-CD last cognitive-distance-list]
      
      ;Upper limit
      ;Special case: Cognitive distance exceeds max-absorptive-capacity
      if last cognitive-distance-list > max-absorptive-capacity
      [set upper-bound-CD max-absorptive-capacity]
      ;Normal case
      if (last cognitive-distance-list > optimal-cognitive-distance) and (last cognitive-distance-list <= max-absorptive-capacity)
      [set upper-bound-CD last cognitive-distance-list]
      
      ;Special case: optimal-cognitive-distance is reached
      if last cognitive-distance-list = optimal-cognitive-distance
      [set lower-bound-CD last cognitive-distance-list
        set upper-bound-CD last cognitive-distance-list]
    ]
  ]
  
  ;Partner selection criteria
  ;Firms which learned, need to update their upper-bound-CD
  if max-AC-increase-current > 0
  [
    ifelse upper-bound-CD = (sqrt ((max-pxcor ^ 2) + (max-pycor ^ 2)))
    []
    [set upper-bound-CD (upper-bound-CD + max-AC-increase-current)]
  ]
  
  ;Update novelty value
  ifelse max-AC-increase-current > 0
  []
  [
    if any? my-in-links
    [let inbound one-of my-in-links
      let transit [novelty-value] of inbound
      set novelty-value-list lput transit novelty-value-list]
    if any? my-out-links
    [let outbound one-of my-out-links
      let transit [novelty-value] of outbound
      set novelty-value-list lput transit novelty-value-list]
  ]
  
  ;Update absorptive capacity
  if partner = true
  [
    ;Absorptive capacity cannot be less than 0
    if last cognitive-distance-list > max-absorptive-capacity
    [set absorptive-capacity 0]
    ;Normal case
    set absorptive-capacity ((-1 * last cognitive-distance-list) + max-absorptive-capacity)
  ]
  
  ;Update satisfaction
  ;The difference between absorptive capacity and novelty value (absolute number) displays
  ;the satisfaction with the relationship
  if partner = true
  [
    ;Firms which have a partner whose cognitive distance is larger than they can bear (in terms
    ;of their (max-)absorptive-capacity), are unsatisfied 
    if last cognitive-distance-list > max-absorptive-capacity
      [set satisfaction -100]
    ;All other firms which are in the range, normally update their values
    if last cognitive-distance-list <= max-absorptive-capacity
      [set satisfaction (absorptive-capacity - last novelty-value-list)]
  ]
  
  ;Update optimal cognitive distance (only firms which learned)
  if max-AC-increase-current > 0
   [set optimal-cognitive-distance max-absorptive-capacity / 2]
  
  ;Update exploratory learning (only firms which learned)
  if max-AC-increase-current > 0
   [set-exploratory-learning]
  
end



;----------------------------------------------
;MODEL SCHEDULE (Observer)
;----------------------------------------------

to go
  set leaving-firms 0
  ask firms [determine-satisfaction]
  ask firms [set-dyadic-learning-probability]
  ask firms [create-knowledge]
  calculate-median
  ask firms [determine-success-and-learn]
  set network-knowledge-created-month 0
  ask firms [search-for-partner]
  ask firms [update-values]
  ask firms [set max-AC-increase-current 0]
  ask firms [be-frustrated-and-exit]
  new-firms-enter
  
  tick
  
  ;Stop condition
  if ticks = 500
    [stop]
  
end



;----------------------------------------------
;SUBMODELS
;----------------------------------------------

;----------------------------------------------
;SUBMODEL 1 "Determine the satisfaction with the
;dyad: Decide to stay or to leave the
;partnership" (Turtle)
;----------------------------------------------

to determine-satisfaction
  
  ;Partnerships do not last longer than 48 time steps (i.e. months, i.e. 4 years)
  if n-rounds-collaboration-this-dyad >= 48
  ;Before firms terminate a partnership, both firms need to write their variables to the output file
  [
    both-firms-write-variables-to-file-satisfied
    cut-partnership
  ]
  
  ;Partners which are already in a satisfying partnership, are not asked again
  if (partner = true) and (satisfied = true)
  [stop]
  
  ;Process for acceptable-satisfaction (global variable)
  if (partner = true) and (satisfied = false) and (firm-satisfaction? = false)
  [
    ;Optimum
    if acceptable-satisfaction = 0
    [
      ;If optimal satisfaction is reached, firm wants to stay with this partner
      if satisfaction = optimal-satisfaction
      [set satisfied true]
      ;If firm is not satisfied (< / > acceptable-satisfaction), it leaves the partnership; by doing so, the other partner
      ;is also forced to do the same
      if satisfaction < acceptable-satisfaction
        [cut-partnership]
      if satisfaction > acceptable-satisfaction
        [cut-partnership]
    ]
    
    ;Exploitation
    if acceptable-satisfaction < 0
    [
      if (satisfaction >= acceptable-satisfaction) and (satisfaction <= 0)
      [set satisfied true]
      if satisfaction < acceptable-satisfaction
        [cut-partnership]
      if satisfaction > 0
        [cut-partnership]
    ]
    ;Exploration
    if acceptable-satisfaction > 0
    [
      if (satisfaction <= acceptable-satisfaction) and (satisfaction >= 0)
      [set satisfied true]
      if satisfaction > acceptable-satisfaction
        [cut-partnership]
      if satisfaction < 0
        [cut-partnership]
    ]
  ]
  
  ;Process for firm-acceptable-satisfaction (firm variable)
  if (partner = true) and (satisfied = false) and (firm-satisfaction? = true)
  [
    ;Optimum
    if firm-acceptable-satisfaction = 0
    [
      if satisfaction = optimal-satisfaction
      [set satisfied true]
      if satisfaction < firm-acceptable-satisfaction
        [cut-partnership]
      if satisfaction > firm-acceptable-satisfaction
        [cut-partnership]
    ]
    
    ;Exploitation
    if firm-acceptable-satisfaction < 0
    [
      if (satisfaction >= firm-acceptable-satisfaction) and (satisfaction <= 0)
      [set satisfied true]
      if satisfaction < firm-acceptable-satisfaction
        [cut-partnership]
      if satisfaction > 0
        [cut-partnership]
    ]
    ;Exploration
    if firm-acceptable-satisfaction > 0
    [
      if (satisfaction <= firm-acceptable-satisfaction) and (satisfaction >= 0)
      [set satisfied true]
      if satisfaction > firm-acceptable-satisfaction
        [cut-partnership]
      if satisfaction < 0
        [cut-partnership]
    ]
  ]

  ;In satisfying partnerships, both partners turn green
  if satisfied = true
  [
    if any? my-in-links
    [let inbound one-of my-in-links
      ask [other-end] of inbound 
      [if satisfied = true
        [set color green
          ;Global variable number-of-satisfied-dyads increases by 1
          set number-of-satisfied-dyads number-of-satisfied-dyads + 1
          ;Firm variable n-satisfying-partners counts number of partners
          ;which are satisfying
          set n-satisfying-partners n-satisfying-partners + 1
          ;Satisfied firms store the value of MaxACdifference of their partnerships
          set max-AC-difference-list lput [max-AC-difference] of inbound max-AC-difference-list
          ask [other-end] of inbound 
          [set color green
            set n-satisfying-partners n-satisfying-partners + 1
            set max-AC-difference-list lput [max-AC-difference] of inbound max-AC-difference-list
          ]]
      ]
    ]
    if any? my-out-links
      [let outbound one-of my-out-links
        ask [other-end] of outbound 
        [if satisfied = true
          [set color green
            set number-of-satisfied-dyads number-of-satisfied-dyads + 1
            set n-satisfying-partners n-satisfying-partners + 1
            set max-AC-difference-list lput [max-AC-difference] of outbound max-AC-difference-list
            ask [other-end] of outbound 
            [set color green
              set n-satisfying-partners n-satisfying-partners + 1
              set max-AC-difference-list lput [max-AC-difference] of outbound max-AC-difference-list
            ]]
        ]
      ]  
  ]
  
end

;----------------------------------------------
;Submodel 1.1 Cut partnership (Turtle)
;----------------------------------------------

to cut-partnership
  set partner false
  set satisfied false
  set satisfaction -100
  set absorptive-capacity 0
  set n-rounds-collaboration-this-dyad 0
  set below-mean 0
  set above-mean 0
  set color blue
  set dyadic-learning-probability 0
  set knowledge-created-month 0
  set CD-category "NA"
  if any? my-in-links
    [let inbound one-of my-in-links
      ask [other-end] of inbound 
        [set partner false
          set satisfied false
          set satisfaction -100
          set absorptive-capacity 0
          set n-rounds-collaboration-this-dyad 0
          set below-mean 0
          set above-mean 0
          set color blue
          set dyadic-learning-probability 0
          set knowledge-created-month 0
          set CD-category "NA"]
      ask inbound [die]
    ]
  if any? my-out-links
    [let outbound one-of my-out-links
      ask [other-end] of outbound 
        [set partner false
          set satisfied false
          set satisfaction -100
          set absorptive-capacity 0
          set n-rounds-collaboration-this-dyad 0
          set below-mean 0
          set above-mean 0
          set color blue
          set dyadic-learning-probability 0
          set knowledge-created-month 0
          set CD-category "NA"]
      ask outbound [die]
    ]
end



;----------------------------------------------
;SUBMODEL 2 "Determine learning probability from
;dyad" (Turtle)
;----------------------------------------------

to set-dyadic-learning-probability
  
  ;The variable dyadic-learning-probability determines, as a function of time, at which rate (probability)
  ;a firm is able to grasp the novel knowledge from the dyad
  
  ;Learning takes place in satisfying partnerships only
  if satisfied = true
  [
    ;Low cognitive distance
    if (last cognitive-distance-list < (optimal-cognitive-distance - (optimal-cognitive-distance * var-opt-CD)))
    and (last cognitive-distance-list > 0)
      [set CD-category "Low"
        set dyadic-learning-probability (0.075 * exp (0.0361375 * n-rounds-collaboration-this-dyad))]
    
    ;Medium/optimal cognitive distance
    if (last cognitive-distance-list >= (optimal-cognitive-distance - (optimal-cognitive-distance * var-opt-CD)))
    and (last cognitive-distance-list <=  (optimal-cognitive-distance + (optimal-cognitive-distance * var-opt-CD)))    
      [set CD-category "Medium"
        set dyadic-learning-probability (0.2 * exp(0.0319057 * n-rounds-collaboration-this-dyad))]
    
    ;High cognitive distance
    if (last cognitive-distance-list > (optimal-cognitive-distance + (optimal-cognitive-distance * var-opt-CD)))
    and (last cognitive-distance-list < max-absorptive-capacity)
      [set CD-category "High"
        set dyadic-learning-probability ((0.00104167 * n-rounds-collaboration-this-dyad) + 0.1)]
    
    ;Extreme values
    if last cognitive-distance-list = 0
      [set CD-category "NA"
        set dyadic-learning-probability 0]
    if last cognitive-distance-list = max-absorptive-capacity
      [set CD-category "NA"
        set dyadic-learning-probability 0]
      
    ;Firms memorize the minimum and maximum values of their dyadic-learning-probability
    if dyadic-learning-probability > max-dyadic-learn-prob
    [set max-dyadic-learn-prob dyadic-learning-probability]
    if dyadic-learning-probability = max-dyadic-learn-prob
    []
    if dyadic-learning-probability < max-dyadic-learn-prob
    [
      if min-dyadic-learn-prob = 0
      [set min-dyadic-learn-prob dyadic-learning-probability]
      if dyadic-learning-probability < min-dyadic-learn-prob
      [set min-dyadic-learn-prob dyadic-learning-probability]
      if dyadic-learning-probability >= min-dyadic-learn-prob
      []
    ]
    
  ]
  
end



;----------------------------------------------
;SUBMODEL 3 "Organizational knowledge creation"
;(Turtle)
;----------------------------------------------

to create-knowledge
  
  ;Knowledge can only be created with a satisfying partner
  if satisfied = true 
  [
    ;The local variable knowledge-grasped determines how much of the novelty value of the dyadic
    ;relationship can be theoretically realized at maximum by the individual firm
    let knowledge-grasped (last novelty-value-list * dyadic-learning-probability)
    
    ;Organizational learning process:
    ;The knowledge-grasped is transformed by every firm individually 
    let result-1 (exploratory-learning * knowledge-grasped)
    let result-2 (transformative-learning * result-1)
    let result-3 (exploitative-learning * result-2)
    
    ;The resulting novel knowledge is saved every time step and accumulated over time
    set knowledge-created-month result-3
    set knowledge-created-total (knowledge-created-total + knowledge-created-month)
    
    ;Firm informs the network about the novelty of its knowledge created
    set network-knowledge-created-month (network-knowledge-created-month + knowledge-created-month)
    set network-knowledge-created-total (network-knowledge-created-total + knowledge-created-month)
    
    ;Number of successful collaboration rounds is increased by 1
    set n-rounds-collaboration-this-dyad n-rounds-collaboration-this-dyad + 1
    set n-rounds-collaboration-total n-rounds-collaboration-total + 1
  ]
  
end



;----------------------------------------------
;SUBMODEL 4 "Calculate the network's monthly 
;created novel knowledge" (Observer)
;----------------------------------------------

to calculate-median
  
  ;Every time step, the median of the network's monthly created novel knowledge is calculated
  ;The median is used as there are few firms which generate very novel knowledge, this skews the mean
   ifelse any? firms
   [set median-netw-knowl-month median [knowledge-created-month] of firms]
   [set median-netw-knowl-month 0]

   
  ;Firm counts (for current partnership) how often it performs below / above the network's mean
  ask firms with [satisfied = true]
    [
      if knowledge-created-month < median-netw-knowl-month
      [set below-mean below-mean + 1]
      if knowledge-created-month >= median-netw-knowl-month
      [set above-mean above-mean + 1]
    ]
  
end



;----------------------------------------------
;SUBMODEL 5 "Determine knowledge creation success
;and corresponding learning takes place"
;(Turtle)
;----------------------------------------------

to determine-success-and-learn
  
  ;Only satisfied firms can determine their knowledge creation success and eventually learn
  if satisfied = true
  [
    ;On the basis, of the network information, every satisfied firm decides at each time step of the evaluation interval
    ;about its individual success in creating novel knowledge   
    
    if member? n-rounds-collaboration-this-dyad time-intervals
    ;Firms check if they perform worse, equal or better than the mean
    [ 
      
      ;-Underperformance
      ;firms cut partnerships which did not meet the expectations
      if below-mean > above-mean
        [
          ;firms balance their search-costs vs. underperforming partnership
          ;inside-options: Opportunity to find a better partner is higher
          ;outside-options: Engagement in network and partnership are sunk costs
          let outside-options (search-costs-threshold - (search-costs-threshold * 0.2))
          let inside-options (search-costs-threshold * 0.2)
          if (search-costs < inside-options) or (search-costs > outside-options) 
          [
            both-firms-write-variables-to-file-satisfied
            cut-partnership
          ]
        ]
      
      ;-Average performance
      ;firms can go on, but do not learn
      if below-mean = above-mean
        []
      
      ;-Above-average performance
      ;firms are able to increase their prior knowledge (maxAC), and thus their overall absorptive capacity
      if below-mean < above-mean
        [
          ;-Learning process: 
          ;Stock of prior knowledge (max-absorptive-capacity) increases exponentially
          set max-AC-increase-current (max-absorptive-capacity * AC-increase-rate)
          
          ifelse max-AC-increase-current + max-absorptive-capacity > sqrt ((max-pxcor ^ 2) + (max-pycor ^ 2))
          [set max-absorptive-capacity sqrt ((max-pxcor ^ 2) + (max-pycor ^ 2))]
          [set max-absorptive-capacity (max-absorptive-capacity + max-AC-increase-current)]
          
          ;A learning firm keeps record of its learning success (over its lifetime in the network)
          set max-AC-increase-total (max-AC-increase-total + max-AC-increase-current)
        ]
          
    ]
  ]
    
end



;----------------------------------------------
;SUBMODEL 6 "Search for a partner with a better 
;cognitive fit" (Turtle)
;----------------------------------------------

to search-for-partner
  
  ;All firms which have already a partner stop the process here
  if partner = true
   [stop]
  
  ;Search for a new partner
  let potential-partner-pool other firms with [partner = false]
  ifelse any? potential-partner-pool
  ;Every round only one potential partner is asked to become a partner
  [let potential-partner one-of potential-partner-pool
    ;Ask only firms to potentially become a new partner, if you haven't been a partner in the past
    ifelse not member? potential-partner partner-list
    ;The potential-partner as well as the asking agent must fulfill the respective search criteria in terms of cognitive distance
      [if (lower-bound-CD < distance potential-partner) and (distance potential-partner < upper-bound-CD) 
        and ([lower-bound-CD] of potential-partner < distance potential-partner) and (distance potential-partner < [upper-bound-CD] of potential-partner)
        [create-link-to potential-partner
          set number-of-dyads number-of-dyads + 1
          ask links [ask both-ends [set partner true]]
        ]
      ]
      []
  ]
  [stop]
  
end



;----------------------------------------------
;SUBMODEL 7 "Update values" (Turtle)
;----------------------------------------------

to update-values
  
  ;All firms which have a (new) partner and are not yet satisfied,
  ;update their values
  if partner = true and satisfied = false
   [update-firm-characteristics]
    
  ;All firms which were able to learn (i.e. increase their Max-AC),
  ;update their values
  if satisfied = true and max-AC-increase-current > 0
   [update-firm-characteristics]
  
end



;----------------------------------------------
;SUBMODEL 8 "Increasing frustration through 
;search costs by not finding a partner and 
;possible exit" (Turtle)
;----------------------------------------------

to be-frustrated-and-exit
  
  ;Firms which do not find a partner, exit finally the network 
    if partner = false
    [set search-costs search-costs + 1
      if search-costs = search-costs-threshold - 2
      [set color yellow]
      if search-costs = search-costs-threshold - 1
      [set color red]
      ;if a firm's search costs reach the search-costs-threshold, firm leaves the network
      if search-costs = search-costs-threshold
      [;Before a firm leaves the network, it reports its output
        write-variables-to-file-frustrated
        set leaving-firms leaving-firms + 1
        set leaving-firms-total leaving-firms-total + 1
        die]
    ]
  
end



;----------------------------------------------
;SUBMODEL 9 "New firms enter the network for those
;which have left the network" (Observer)
;----------------------------------------------

to new-firms-enter
  
      ;if there are after 12 time steps (i.e. one year) no partnerships with satisfied partners, no new firms will enter the network
      ifelse (ticks >= 12) and (not any? firms with [satisfied = true])
      []
      [
        if leaving-firms > 0
        ;only if firms have left the network, new firms can enter
  
        [if any? firms with [(partner = false) and (search-costs < (search-costs-threshold - 1))]
          ;only if there are still firms which are searching for a partner, new firms can enter
          
          [create-firms leaving-firms 
            [setup-firm-characteristics]
          
          set new-entrants new-entrants + leaving-firms]
        ]
      ]
  
end



;----------------------------------------------
;REPORTERS - Model
;----------------------------------------------

;Cognitive distance (Link)
to-report cognitive-distance
  report link-length
end

;Novelty value (Link)
;Novelty value increases by x with increasing cognitive distance
to-report novelty-value
  report cognitive-distance
end



;----------------------------------------------
;ADMINISTRATION
;----------------------------------------------

;----------------------------------------------
;ADMINISTRATION-1 "Output file for frustrated 
;firms which leave the network"
;----------------------------------------------
;Administration 1.1 (Observer)
;----------------------------------------------
to write-headings-to-file-frustrated
  
  if (file-exists? "Model_InnoNetw_Output_Frustrated.csv")
  [carefully
    [file-delete "Model_InnoNetw_Output_Frustrated.csv"]
    [print error-message]]
  file-open "Model_InnoNetw_Output_Frustrated.csv"
  
  ;Variable names (headings)
  file-type "Tick,"                     ;1
  file-type "Satisfied,"                ;2
  file-type "Firm,"                     ;3
  file-type "Partner,"                  ;4
  file-type "Optimal-CD,"               ;5
  file-type "Lower-bound-CD,"           ;6
  file-type "Upper-bound-CD,"           ;7
  file-type "CD,"                       ;8
  file-type "NV,"                       ;9
  file-type "AC,"                       ;10
  file-type "Max-AC,"                   ;11
  file-type "Max-AC-difference,"        ;12
  file-type "Satisfaction,"             ;13
  file-type "Search-costs,"             ;14
  file-type "Dyadic-learn-prob,"        ;15
  file-type "Exploratory-L,"            ;16
  file-type "Transformative-L,"         ;17
  file-type "Exploitative-L,"           ;18
  file-type "Rel-Efficiency,"           ;19
  file-type "N-rounds-coll-this-dyad,"  ;20
  file-type "Mean-knowledge-created,"   ;21
  file-type "N-rounds-coll,"            ;22
  file-type "Knowledge-created-total,"  ;23
  file-type "N-satisfying-partners,"    ;24
  file-type "Below mean,"               ;25
  file-type "Above mean,"               ;26
  file-type "Median-netw-knowl-month,"  ;27
  file-type "Max-AC-increase-current,"  ;28
  file-type "Max-AC-incrase-total,"     ;29
  file-type "CD-category,"              ;30
  file-type "Min-dyadic-learn-prob,"    ;31
  file-print "Max-dyadic-learn-prob"    ;32
  
  file-close
  
end

;----------------------------------------------
;Administration 1.2 (Turtle)
;----------------------------------------------
to write-variables-to-file-frustrated

  file-open "Model_InnoNetw_Output_Frustrated.csv"
  
  ;Variables
    file-type word ticks ","                                                                     ;1
    file-type word satisfied ","                                                                 ;2
    file-type word who ","                                                                       ;3
    ifelse satisfaction = -100                                                                   ;4
      [file-type word "NA" ","]
      [file-type word [who] of last partner-list ","]
    file-type word optimal-cognitive-distance ","                                                ;5
    file-type word lower-bound-CD ","                                                            ;6
    file-type word upper-bound-CD ","                                                            ;7
    ifelse satisfaction = -100                                                                   ;8
      [file-type word "NA" ","]
      [file-type word last cognitive-distance-list ","]
    ifelse satisfaction = -100                                                                   ;9
      [file-type word "NA" ","]
      [file-type word last novelty-value-list ","]
    file-type word absorptive-capacity ","                                                       ;10
    file-type word max-absorptive-capacity ","                                                   ;11
    ifelse satisfaction = -100                                                                   ;12
      [file-type word "NA" ","]
      [if any? my-in-links
        [ask one-of my-in-links [file-type word max-AC-difference ","]]
      if any? my-out-links
      [ask one-of my-out-links [file-type word max-AC-difference ","]]]
    file-type word satisfaction ","                                                              ;13
    file-type word search-costs ","                                                              ;14
    file-type word dyadic-learning-probability ","                                               ;15
    file-type word exploratory-learning ","                                                      ;16
    file-type word transformative-learning ","                                                   ;17
    file-type word exploitative-learning ","                                                     ;18
    file-type word relative-efficiency ","                                                       ;19
    file-type word n-rounds-collaboration-this-dyad ","                                          ;20
    ifelse n-rounds-collaboration-total = 0                                                      ;21
      [file-type word 0 ","]
      [file-type word (knowledge-created-total / n-rounds-collaboration-total) ","]
    file-type word n-rounds-collaboration-total ","                                              ;22
    file-type word knowledge-created-total ","                                                   ;23
    file-type word n-satisfying-partners ","                                                     ;24
    file-type word below-mean ","                                                                ;25
    file-type word above-mean ","                                                                ;26
    file-type word median-netw-knowl-month ","                                                   ;27
    file-type word max-AC-increase-current ","                                                   ;28
    file-type word max-AC-increase-total ","                                                     ;29
    file-type word CD-category ","                                                               ;30
    file-type word min-dyadic-learn-prob ","                                                     ;31
    file-print max-dyadic-learn-prob                                                             ;32

  file-close
  
end



;----------------------------------------------
;ADMINISTRATION-2 "Output file for satisfied
;firms (each tick)"
;----------------------------------------------
;Administration 2.1 (Observer)
;----------------------------------------------
to write-headings-to-file-satisfied
  
  if (file-exists? "Model_InnoNetw_Output_Satisfied.csv")
  [carefully
    [file-delete "Model_InnoNetw_Output_Satisfied.csv"]
    [print error-message]]
  file-open "Model_InnoNetw_Output_Satisfied.csv"
  
  ;Variable names (headings)
  file-type "Tick,"                     ;1
  file-type "Satisfied,"                ;2
  file-type "Firm,"                     ;3
  file-type "Partner,"                  ;4
  file-type "Optimal-CD,"               ;5
  file-type "Lower-bound-CD,"           ;6
  file-type "Upper-bound-CD,"           ;7
  file-type "CD,"                       ;8
  file-type "NV,"                       ;9
  file-type "AC,"                       ;10
  file-type "Max-AC,"                   ;11
  file-type "Max-AC-difference,"        ;12
  file-type "Satisfaction,"             ;13
  file-type "Search-costs,"             ;14
  file-type "Dyadic-learn-prob,"        ;15
  file-type "Exploratory-L,"            ;16
  file-type "Transformative-L,"         ;17
  file-type "Exploitative-L,"           ;18
  file-type "Rel-Efficiency,"           ;19
  file-type "N-rounds-coll-this-dyad,"  ;20
  file-type "Mean-knowledge-created,"   ;21
  file-type "N-rounds-coll,"            ;22
  file-type "Knowledge-created-total,"  ;23
  file-type "N-satisfying-partners,"    ;24
  file-type "Below mean,"               ;25
  file-type "Above mean,"               ;26
  file-type "Median-netw-knowl-month,"  ;27
  file-type "Max-AC-increase-current,"  ;28
  file-type "Max-AC-increase-total,"    ;29
  file-type "CD-category,"              ;30
  file-type "Min-dyadic-learn-prob,"    ;31
  file-print "Max-dyadic-learn-prob"    ;32
  
  file-close
  
end

;----------------------------------------------
;Administration 2.2 (Turtle)
;----------------------------------------------
to write-variables-to-file-satisfied

  file-open "Model_InnoNetw_Output_Satisfied.csv"
  
  ;Variables
    file-type word ticks ","                                                                     ;1
    file-type word satisfied ","                                                                 ;2
    file-type word who ","                                                                       ;3
    ifelse satisfaction = -100                                                                   ;4
      [file-type word "NA" ","]
      [ifelse last partner-list = nobody
        [file-type word "NA" ","]
        [file-type word [who] of last partner-list ","]]
    file-type word optimal-cognitive-distance ","                                                ;5
    file-type word lower-bound-CD ","                                                            ;6
    file-type word upper-bound-CD ","                                                            ;7
    ifelse satisfaction = -100                                                                   ;8
      [file-type word "NA" ","]
      [file-type word last cognitive-distance-list ","]
    ifelse satisfaction = -100                                                                   ;9
      [file-type word "NA" ","]
      [file-type word last novelty-value-list ","]
    file-type word absorptive-capacity ","                                                       ;10
    file-type word max-absorptive-capacity ","                                                   ;11
    ifelse satisfaction = -100                                                                   ;12
      [file-type word "NA" ","]
      [if any? my-in-links
        [ask one-of my-in-links [file-type word max-AC-difference ","]]
      if any? my-out-links
      [ask one-of my-out-links [file-type word max-AC-difference ","]]]
    file-type word satisfaction ","                                                              ;13
    file-type word search-costs ","                                                              ;14
    file-type word dyadic-learning-probability ","                                               ;15
    file-type word exploratory-learning ","                                                      ;16
    file-type word transformative-learning ","                                                   ;17
    file-type word exploitative-learning ","                                                     ;18
    file-type word relative-efficiency ","                                                       ;19
    file-type word n-rounds-collaboration-this-dyad ","                                          ;20
    ifelse n-rounds-collaboration-total = 0                                                      ;21
      [file-type word 0 ","]
      [file-type word (knowledge-created-total / n-rounds-collaboration-total) ","]
    file-type word n-rounds-collaboration-total ","                                              ;22
    file-type word knowledge-created-total ","                                                   ;23
    file-type word n-satisfying-partners ","                                                     ;24
    file-type word below-mean ","                                                                ;25
    file-type word above-mean ","                                                                ;26
    file-type word median-netw-knowl-month ","                                                   ;27
    file-type word max-AC-increase-current ","                                                   ;28
    file-type word max-AC-increase-total ","                                                     ;29
    file-type word CD-category ","                                                               ;30
    file-type word min-dyadic-learn-prob ","                                                     ;31
    file-print max-dyadic-learn-prob                                                             ;32

  file-close
  
end

;----------------------------------------------
;Administration 2.3 (Turtle)
;----------------------------------------------
to both-firms-write-variables-to-file-satisfied
  
  write-variables-to-file-satisfied
    if any? my-in-links
    [let inbound one-of my-in-links
      ask [other-end] of inbound 
        [write-variables-to-file-satisfied]
    ]
  if any? my-out-links
    [let outbound one-of my-out-links
      ask [other-end] of outbound 
        [write-variables-to-file-satisfied]
    ]
  
end



;----------------------------------------------
;ADMINISTRATION-3 Reporters
;for model analysis
;----------------------------------------------

;Write percentage of dyads to interface (Observer)
to-report dyad-percentage
  if any? firms
    [let general-percentage ((count firms with [partner = true]) / count firms)
      report general-percentage]
end

;Write percentage of satisfied dyads to interface (Observer)
to-report satisfied-percentage
  if any? firms
  [let specific-percentage ((count firms with [satisfied = true]) / count firms)
    report specific-percentage]
end

;Firm's relative efficiency
;in organizational knowledge creation (Turtle)
to-report relative-efficiency
  report exploratory-learning * transformative-learning * exploitative-learning
end

;Firm's overall mean knowledge created
;(over all partners) (Turtle)
to-report mean-knowledge-created
  report knowledge-created-total / n-rounds-collaboration-total
end
@#$#@#$#@
GRAPHICS-WINDOW
269
21
761
534
-1
-1
22.9524
1
10
1
1
1
0
0
0
1
0
20
0
20
0
0
1
ticks
30.0

BUTTON
37
24
101
57
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
38
65
101
98
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
109
65
188
98
Go Once
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
838
11
1240
237
Frequency distribution of dyads according to cognitive distance
Cognitive Distance
Number of Dyads
0.0
14.0
0.0
6.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [cognitive-distance] of links"

SLIDER
34
276
212
309
acceptable-satisfaction
acceptable-satisfaction
-28
28
-28
0.1
1
NIL
HORIZONTAL

TEXTBOX
38
197
188
253
Satisfaction level with relationship where firm stops searching for a better partner
11
0.0
1

SLIDER
37
125
209
158
network-members
network-members
10
200
125
1
1
NIL
HORIZONTAL

PLOT
838
242
1240
417
Frequency distribution of firms according to their max-absorptive-capacity
max-absorptive-capacity
Number of firms
0.0
28.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [max-absorptive-capacity] of firms"

SLIDER
34
396
206
429
search-costs-threshold
search-costs-threshold
5
50
50
1
1
NIL
HORIZONTAL

PLOT
837
422
1239
592
Frequency distribution of dyads according to their Max-AC-Difference
max-AC-difference
Number of dyads
0.0
15.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [max-AC-difference] of links"

TEXTBOX
39
178
189
196
SATISFACTION:
12
0.0
1

TEXTBOX
36
374
186
392
Search costs
11
0.0
1

TEXTBOX
37
251
225
279
Exploration area - Exploitation area
11
0.0
1

SWITCH
35
321
185
354
firm-satisfaction?
firm-satisfaction?
1
1
-1000

PLOT
838
600
1237
771
Number of satisfied dyads
Time
No. satisfied dyads
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot number-of-satisfied-dyads"

PLOT
1250
12
1626
237
Knowledge-created-month (Satisfied firms)
Knowledge-created
Frequency
0.0
3.0
0.0
10.0
true
false
"set-histogram-num-bars 20" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [knowledge-created-month] of firms with [satisfied = true]"

PLOT
1248
244
1626
419
Knowledge-created-total (All firms)
Time
Knowledge-created-total
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot network-knowledge-created-total"

PLOT
1249
424
1627
595
"Learners" - Fiirms with increase in MaxAC
Max-AC-increase
Frequency
0.0
5.0
0.0
10.0
true
false
"set-histogram-num-bars 20" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [max-AC-increase-total] of firms"

CHOOSER
426
621
564
666
var-opt-CD
var-opt-CD
0.05 0.1 0.15
0

TEXTBOX
274
620
424
662
Inter-organizational learning: Allowed deviation from optimal cognitive distance
11
0.0
1

CHOOSER
31
505
211
550
exploratory-learning-setting
exploratory-learning-setting
"linear" "quadratic"
0

TEXTBOX
35
442
185
498
Relationship between prior knowledge (maxAC) and a firm's exploratory learning capability
11
0.0
1

CHOOSER
31
607
169
652
AC-increase-rate
AC-increase-rate
0.0010 0.01 0.05 0.1 0.15
0

TEXTBOX
32
572
182
600
Rate of increase of absorptive capacity over time
11
0.0
1

CHOOSER
428
545
566
590
evaluation-interval
evaluation-interval
"quarterly" "half-year" "yearly" "two-year"
0

TEXTBOX
274
553
424
581
Determines firms' performance evaluation intervals
11
0.0
1

MONITOR
626
559
801
604
Percentage of firms with partner
dyad-percentage
17
1
11

MONITOR
627
611
801
656
Percentage of satisfied firms
satisfied-percentage
17
1
11

@#$#@#$#@
## Model Innovation Network

This agent-based model represents a stylized inter-organizational innovation network where firms collaborate with each other in order to generate novel organizational knowledge. The focus of this agent-based model lies on the firm’s partner choice and the consequent inter-organizational collaboration and learning process to access new knowledge for the purpose of organizational knowledge creation. These processes are largely influenced by the cognitive distance between the firms, their relative absorptive capacity as well as their organizational absorptive capacity.

The model addresses the following main research question:
How does absorptive capacity influence a firm’s collaboration partner choice, the subsequent inter-organizational learning process, as well as the organizational learning process and which effect has organizational learning in turn on the development of absorptive capacity?

A detailed description of the model can be found in:
Schmid, S. (2015), Organizational Learning in Innovation Networks: Exploring the Role of Cognitive Distance and Absorptive Capacity. An Agent-Based Model, Metropolis, Marburg.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
